@@ -19,14 +19,13 @@
 #include <apt-pkg/deblistparser.h>
 #include <apt-pkg/pkgcache.h>
 #include <apt-pkg/tagfile.h>
-#include <apt-pkg/md5.h>
-#include <apt-pkg/sha1.h>
-#include <apt-pkg/sha2.h>
+#include <apt-pkg/hashes.h>
 #include <apt-pkg/init.h>
 #include <apt-pkg/pkgsystem.h>
 #include <apt-pkg/orderlist.h>
 #include <apt-pkg/aptconfiguration.h>
 #include <apt-pkg/fileutl.h>
+#include <apt-pkg/gpgv.h>
 
 #include <sys/stat.h>
 #include <libintl.h>
@@ -34,6 +33,30 @@
 #include <Python.h>
 									/*}}}*/
 
+static char PyAptError_Doc[] =
+   "Exception class for most python-apt exceptions.\n"
+   "\n"
+   "This class replaces the use of :class:`SystemError` in previous versions\n"
+   "of python-apt. It inherits from :class:`SystemError`, so make sure to\n"
+   "catch this class first.\n\n"
+   ".. versionadded:: 1.1";
+
+PyObject *PyAptError;
+
+static char PyAptWarning_Doc[] =
+   "Exception class for most python-apt warnings.\n"
+   "\n"
+   ".. versionadded:: 2.7";
+
+PyObject *PyAptWarning;
+
+static char PyAptCacheMismatchError_Doc[] =
+   "Raised when passing an object from a different cache to\n"
+   ":class:`apt_pkg.DepCache` methods\n\n"
+   ".. versionadded:: 1.6.1";
+
+PyObject *PyAptCacheMismatchError;
+
 /**
  * A Python->C->Python gettext() function.
  *
@@ -48,7 +71,7 @@ static PyObject *py_gettext(PyObject *self, PyObject *Args) {
     if (PyArg_ParseTuple(Args,"s|s:gettext",&msg, &domain) == 0)
         return 0;
 
-    return PyString_FromString(dgettext(domain, msg));
+    return CppPyString(dgettext(domain, msg));
 }
 
 // newConfiguration - Build a new configuration class			/*{{{*/
@@ -66,8 +89,8 @@ static PyObject *VersionCompare(PyObject *Self,PyObject *Args)
 {
    char *A;
    char *B;
-   int LenA;
-   int LenB;
+   Py_ssize_t LenA;
+   Py_ssize_t LenB;
 
    if (PyArg_ParseTuple(Args,"s#s#",&A,&LenA,&B,&LenB) == 0)
       return 0;
@@ -103,7 +126,7 @@ static PyObject *CheckDep(PyObject *Self,PyObject *Args)
    if (strcmp(OpStr, "<") == 0) OpStr = "<<";
    if (*debListParser::ConvertRelation(OpStr,Op) != 0)
    {
-      PyErr_SetString(PyExc_ValueError,"Bad comparision operation");
+      PyErr_SetString(PyExc_ValueError,"Bad comparison operation");
       return 0;
    }
 
@@ -129,7 +152,7 @@ static PyObject *UpstreamVersion(PyObject *Self,PyObject *Args)
 }
 
 static const char *doc_ParseDepends =
-"parse_depends(s: str[, strip_multi_arch : bool = True]) -> list\n"
+"parse_depends(s: str[, strip_multi_arch : bool = True[, architecture : string]]) -> list\n"
 "\n"
 "Parse the dependencies given by 's' and return a list of lists. Each of\n"
 "these lists represents one or more options for an 'or' dependency in\n"
@@ -138,10 +161,12 @@ static const char *doc_ParseDepends =
 "if no version was requested. The element 'ver' is a comparison\n"
 "operator ('<', '<=', '=', '>=', or '>').\n\n"
 "If 'strip_multi_arch' is True, :any (and potentially other special values)\n"
-"will be stripped from the full package name";
+"will be stripped from the full package name"
+"The 'architecture' parameter may be used to specify a non-native architecture\n"
+"for the dependency parsing.";
 
 static const char *parse_src_depends_doc =
-"parse_src_depends(s: str[, strip_multi_arch : bool = True]) -> list\n"
+"parse_src_depends(s: str[, strip_multi_arch : bool = True[, architecture : string]]) -> list\n"
 "\n"
 "Parse the dependencies given by 's' and return a list of lists. Each of\n"
 "these lists represents one or more options for an 'or' dependency in\n"
@@ -154,8 +179,10 @@ static const char *parse_src_depends_doc =
 "only contains those dependencies for the architecture set in the\n"
 "configuration variable APT::Architecture\n\n"
 "If 'strip_multi_arch' is True, :any (and potentially other special values)\n"
-"will be stripped from the full package name";
-static PyObject *RealParseDepends(PyObject *Self,PyObject *Args,
+"will be stripped from the full package name"
+"The 'architecture' parameter may be used to specify a non-native architecture\n"
+"for the dependency parsing.";
+static PyObject *RealParseDepends(PyObject *Self,PyObject *Args,PyObject *Kwds,
                                   bool ParseArchFlags, bool ParseRestrictionsList,
                                   std::string name, bool debStyle=false)
 {
@@ -166,10 +193,12 @@ static PyObject *RealParseDepends(PyObject *Self,PyObject *Args,
 
    const char *Start;
    const char *Stop;
-   int Len;
+   Py_ssize_t Len;
+   const char *Arch = NULL;
+   char *kwlist[] = {"s", "strip_multi_arch", "architecture", 0};
 
-   if (PyArg_ParseTuple(Args,(char *)("s#|b:" + name).c_str(), 
-                        &Start, &Len, &StripMultiArch) == 0)
+   if (PyArg_ParseTupleAndKeywords(Args,Kwds,(char *)("s#|bs:" + name).c_str(), kwlist,
+                        &Start, &Len, &StripMultiArch, &Arch) == 0)
       return 0;
    Stop = Start + Len;
    PyObject *List = PyList_New(0);
@@ -178,10 +207,15 @@ static PyObject *RealParseDepends(PyObject *Self,PyObject *Args,
    {
       if (Start == Stop)
 	 break;
+      if (Arch == NULL)
+	 Start = debListParser::ParseDepends(Start,Stop,Package,Version,Op,
+					     ParseArchFlags, StripMultiArch,
+					     ParseRestrictionsList);
+      else
+	 Start = debListParser::ParseDepends(Start,Stop,Package,Version,Op,
+					     ParseArchFlags, StripMultiArch,
+                                             ParseRestrictionsList, Arch);
 
-      Start = debListParser::ParseDepends(Start,Stop,Package,Version,Op,
-					  ParseArchFlags, StripMultiArch,
-                                          ParseRestrictionsList);
       if (Start == 0)
       {
 	 PyErr_SetString(PyExc_ValueError,"Problem Parsing Dependency");
@@ -212,13 +246,13 @@ static PyObject *RealParseDepends(PyObject *Self,PyObject *Args,
    }
    return List;
 }
-static PyObject *ParseDepends(PyObject *Self,PyObject *Args)
+static PyObject *ParseDepends(PyObject *Self,PyObject *Args, PyObject *Kwds)
 {
-   return RealParseDepends(Self, Args, false, false, "parse_depends");
+   return RealParseDepends(Self, Args, Kwds, false, false, "parse_depends");
 }
-static PyObject *ParseSrcDepends(PyObject *Self,PyObject *Args)
+static PyObject *ParseSrcDepends(PyObject *Self,PyObject *Args, PyObject *Kwds)
 {
-   return RealParseDepends(Self, Args, true, true, "parse_src_depends");
+   return RealParseDepends(Self, Args, Kwds, true, true, "parse_src_depends");
 }
 									/*}}}*/
 // md5sum - Compute the md5sum of a file or string			/*{{{*/
@@ -235,31 +269,35 @@ static PyObject *md5sum(PyObject *Self,PyObject *Args)
    if (PyArg_ParseTuple(Args,"O",&Obj) == 0)
       return 0;
 
+   if (PyErr_WarnEx(PyExc_DeprecationWarning,
+                    "apt_pkg.md5sum is deprecated, use apt_pkg.Hashes", 1) == -1)
+        return NULL;
+
    // Digest of a string.
    if (PyBytes_Check(Obj) != 0)
    {
       char *s;
       Py_ssize_t len;
-      MD5Summation Sum;
+      Hashes Sum(Hashes::MD5SUM);
       PyBytes_AsStringAndSize(Obj, &s, &len);
       Sum.Add((const unsigned char*)s, len);
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::MD5SUM).HashValue());
    }
 
    // Digest of a file
    int Fd = PyObject_AsFileDescriptor(Obj);
    if (Fd != -1)
    {
-      MD5Summation Sum;
+      Hashes Sum(Hashes::MD5SUM);
       struct stat St;
       if (fstat(Fd,&St) != 0 ||
 	  Sum.AddFD(Fd,St.st_size) == false)
       {
-	 PyErr_SetFromErrno(PyExc_SystemError);
+	 PyErr_SetFromErrno(PyAptError);
 	 return 0;
       }
 
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::MD5SUM).HashValue());
    }
 
    PyErr_SetString(PyExc_TypeError,"Only understand strings and files");
@@ -280,38 +318,42 @@ static PyObject *sha1sum(PyObject *Self,PyObject *Args)
    if (PyArg_ParseTuple(Args,"O",&Obj) == 0)
       return 0;
 
+   if (PyErr_WarnEx(PyExc_DeprecationWarning,
+                    "apt_pkg.sha1sum is deprecated, use apt_pkg.Hashes", 1) == -1)
+        return NULL;
+
    // Digest of a string.
    if (PyBytes_Check(Obj) != 0)
    {
       char *s;
       Py_ssize_t len;
-      SHA1Summation Sum;
+      Hashes Sum(Hashes::SHA1SUM);
       PyBytes_AsStringAndSize(Obj, &s, &len);
       Sum.Add((const unsigned char*)s, len);
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::SHA1SUM).HashValue());
    }
 
    // Digest of a file
    int Fd = PyObject_AsFileDescriptor(Obj);
    if (Fd != -1)
    {
-      SHA1Summation Sum;
+      Hashes Sum(Hashes::SHA1SUM);
       struct stat St;
       if (fstat(Fd,&St) != 0 ||
 	  Sum.AddFD(Fd,St.st_size) == false)
       {
-	 PyErr_SetFromErrno(PyExc_SystemError);
+	 PyErr_SetFromErrno(PyAptError);
 	 return 0;
       }
 
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::SHA1SUM).HashValue());
    }
 
    PyErr_SetString(PyExc_TypeError,"Only understand strings and files");
    return 0;
 }
 									/*}}}*/
-// sha256sum - Compute the sha1sum of a file or string			/*{{{*/
+// sha256sum - Compute the sha256sum of a file or string		/*{{{*/
 // ---------------------------------------------------------------------
 static const char *doc_sha256sum =
     "sha256sum(object) -> str\n\n"
@@ -325,31 +367,84 @@ static PyObject *sha256sum(PyObject *Self,PyObject *Args)
    if (PyArg_ParseTuple(Args,"O",&Obj) == 0)
       return 0;
 
+   if (PyErr_WarnEx(PyExc_DeprecationWarning,
+                    "apt_pkg.sha256sum is deprecated, use apt_pkg.Hashes", 1) == -1)
+        return NULL;
+
    // Digest of a string.
    if (PyBytes_Check(Obj) != 0)
    {
       char *s;
       Py_ssize_t len;
-      SHA256Summation Sum;
+      Hashes Sum(Hashes::SHA256SUM);
       PyBytes_AsStringAndSize(Obj, &s, &len);
       Sum.Add((const unsigned char*)s, len);
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::SHA256SUM).HashValue());
    }
 
    // Digest of a file
    int Fd = PyObject_AsFileDescriptor(Obj);
    if (Fd != -1)
    {
-      SHA256Summation Sum;
+      Hashes Sum(Hashes::SHA256SUM);
       struct stat St;
       if (fstat(Fd,&St) != 0 ||
 	  Sum.AddFD(Fd,St.st_size) == false)
       {
-	 PyErr_SetFromErrno(PyExc_SystemError);
+	 PyErr_SetFromErrno(PyAptError);
 	 return 0;
       }
 
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::SHA256SUM).HashValue());
+   }
+
+   PyErr_SetString(PyExc_TypeError,"Only understand strings and files");
+   return 0;
+}
+									/*}}}*/
+// sha512sum - Compute the sha512sum of a file or string		/*{{{*/
+// ---------------------------------------------------------------------
+static const char *doc_sha512sum =
+    "sha512sum(object) -> str\n\n"
+    "Return the sha512sum of the object. 'object' may either be a string, in\n"
+    "which case the sha512sum of the string is returned, or a file() object\n"
+    "(or file descriptor), in which case the sha512sum of its contents is\n"
+    "returned.";;
+static PyObject *sha512sum(PyObject *Self,PyObject *Args)
+{
+   PyObject *Obj;
+   if (PyArg_ParseTuple(Args,"O",&Obj) == 0)
+      return 0;
+
+   if (PyErr_WarnEx(PyExc_DeprecationWarning,
+                    "apt_pkg.sha512sum is deprecated, use apt_pkg.Hashes", 1) == -1)
+        return NULL;
+
+   // Digest of a string.
+   if (PyBytes_Check(Obj) != 0)
+   {
+      char *s;
+      Py_ssize_t len;
+      Hashes Sum(Hashes::SHA512SUM);
+      PyBytes_AsStringAndSize(Obj, &s, &len);
+      Sum.Add((const unsigned char*)s, len);
+      return CppPyString(Sum.GetHashString(Hashes::SHA512SUM).HashValue());
+   }
+
+   // Digest of a file
+   int Fd = PyObject_AsFileDescriptor(Obj);
+   if (Fd != -1)
+   {
+      Hashes Sum(Hashes::SHA512SUM);
+      struct stat St;
+      if (fstat(Fd,&St) != 0 ||
+	  Sum.AddFD(Fd,St.st_size) == false)
+      {
+	 PyErr_SetFromErrno(PyAptError);
+	 return 0;
+      }
+
+      return CppPyString(Sum.GetHashString(Hashes::SHA512SUM).HashValue());
    }
 
    PyErr_SetString(PyExc_TypeError,"Only understand strings and files");
@@ -427,6 +522,25 @@ static PyObject *InitSystem(PyObject *Self,PyObject *Args)
    return HandleErrors(Py_None);
 }
 									/*}}}*/
+// gpgv.cc:OpenMaybeClearSignedFile					/*{{{*/
+// ---------------------------------------------------------------------
+static char *doc_OpenMaybeClearSignedFile =
+"open_maybe_clear_signed_file(file: str) -> int\n\n"
+"Open a file and ignore a PGP clear signature.\n"
+"Return a open file descriptor or a error.";
+static PyObject *PyOpenMaybeClearSignedFile(PyObject *Self,PyObject *Args)
+{
+   PyApt_Filename file;
+   char errors = false;
+   if (PyArg_ParseTuple(Args,"O&",PyApt_Filename::Converter, &file,&errors) == 0)
+      return 0;
+
+   FileFd Fd;
+   if (OpenMaybeClearSignedFile(file, Fd) == false)
+      return HandleErrors(MkPyNumber(-1));
+
+   return HandleErrors(MkPyNumber(dup(Fd.Fd())));
+}
 
 // fileutils.cc: GetLock						/*{{{*/
 // ---------------------------------------------------------------------
@@ -452,8 +566,8 @@ static PyObject *GetLock(PyObject *Self,PyObject *Args)
 
 static char *doc_PkgSystemLock =
 "pkgsystem_lock() -> bool\n\n"
-"Acquire the global lock for the package system by using /var/lib/dpkg/lock\n"
-"to do the locking. From Python 2.6 on, the apt_pkg.SystemLock context\n"
+"Acquire the global lock for the package system by using /var/lib/dpkg/lock-frontend\n"
+"and /var/lib/dpkg/lock to do the locking. From Python 2.6 on, the apt_pkg.SystemLock context\n"
 "manager is available and should be used instead.";
 static PyObject *PkgSystemLock(PyObject *Self,PyObject *Args)
 {
@@ -476,6 +590,57 @@ static PyObject *PkgSystemUnLock(PyObject *Self,PyObject *Args)
 
    bool res = _system->UnLock();
 
+   Py_INCREF(Py_None);
+   return HandleErrors(PyBool_FromLong(res));
+}
+
+static char *doc_PkgSystemLockInner =
+"pkgsystem_lock_inner() -> bool\n\n"
+"Reacquire the dpkg 'lock' lock file. Must be called only after\n"
+":meth:`pkgsystem_unlock_inner` and only around invocations of dpkg.\n"
+"\n"
+".. versionadded:: 1.7";
+static PyObject *PkgSystemLockInner(PyObject *Self,PyObject *Args)
+{
+   if (PyArg_ParseTuple(Args,"") == 0)
+      return 0;
+
+   bool res = _system->LockInner();
+
+   Py_INCREF(Py_None);
+   return HandleErrors(PyBool_FromLong(res));
+}
+
+static char *doc_PkgSystemUnLockInner =
+"pkgsystem_unlock_inner() -> bool\n\n"
+"Release the dpkg lock file 'lock'. To be called before manually\n"
+"invoking dpkg.\n"
+"\n"
+".. versionadded:: 1.7";
+static PyObject *PkgSystemUnLockInner(PyObject *Self,PyObject *Args)
+{
+   if (PyArg_ParseTuple(Args,"") == 0)
+      return 0;
+
+   bool res = _system->UnLockInner();
+
+   Py_INCREF(Py_None);
+   return HandleErrors(PyBool_FromLong(res));
+}
+
+static char *doc_PkgSystemIsLocked =
+"pkgsystem_is_locked() -> bool\n\n"
+"Check if the system is locked. Can be used to check whether the inner\n"
+"lock needs to be released or not in generic code.\n"
+"\n"
+".. versionadded:: 1.7";
+static PyObject *PkgSystemIsLocked(PyObject *Self,PyObject *Args)
+{
+   if (PyArg_ParseTuple(Args,"") == 0)
+      return 0;
+
+   bool res = _system->IsLocked();
+
    Py_INCREF(Py_None);
    return HandleErrors(PyBool_FromLong(res));
 }
@@ -499,12 +664,17 @@ static PyMethodDef methods[] =
     "and only does translations after setlocale() has been called."},
 
    // Tag File
-   {"rewrite_section",RewriteSection,METH_VARARGS,doc_RewriteSection},
+
+   {"open_maybe_clear_signed_file",PyOpenMaybeClearSignedFile,METH_VARARGS,
+    doc_OpenMaybeClearSignedFile},
 
    // Locking
    {"get_lock",GetLock,METH_VARARGS,doc_GetLock},
    {"pkgsystem_lock",PkgSystemLock,METH_VARARGS,doc_PkgSystemLock},
    {"pkgsystem_unlock",PkgSystemUnLock,METH_VARARGS,doc_PkgSystemUnLock},
+   {"pkgsystem_lock_inner",PkgSystemLockInner,METH_VARARGS,doc_PkgSystemLockInner},
+   {"pkgsystem_unlock_inner",PkgSystemUnLockInner,METH_VARARGS,doc_PkgSystemUnLockInner},
+   {"pkgsystem_is_locked",PkgSystemIsLocked,METH_VARARGS,doc_PkgSystemIsLocked},
 
    // Command line
    {"read_config_file",LoadConfig,METH_VARARGS,doc_LoadConfig},
@@ -518,13 +688,14 @@ static PyMethodDef methods[] =
    {"upstream_version",UpstreamVersion,METH_VARARGS,doc_UpstreamVersion},
 
    // Depends
-   {"parse_depends",ParseDepends,METH_VARARGS,doc_ParseDepends},
-   {"parse_src_depends",ParseSrcDepends,METH_VARARGS,parse_src_depends_doc},
+   {"parse_depends",reinterpret_cast<PyCFunction>(static_cast<PyCFunctionWithKeywords>(ParseDepends)),METH_VARARGS|METH_KEYWORDS,doc_ParseDepends},
+   {"parse_src_depends",reinterpret_cast<PyCFunction>(static_cast<PyCFunctionWithKeywords>(ParseSrcDepends)),METH_VARARGS|METH_KEYWORDS,parse_src_depends_doc},
 
-   // Stuff
+   // Hashes
    {"md5sum",md5sum,METH_VARARGS,doc_md5sum},
    {"sha1sum",sha1sum,METH_VARARGS,doc_sha1sum},
    {"sha256sum",sha256sum,METH_VARARGS,doc_sha256sum},
+   {"sha512sum",sha512sum,METH_VARARGS,doc_sha512sum},
 
    // multiarch
    {"get_architectures", GetArchitectures, METH_VARARGS, doc_GetArchitectures},
@@ -629,9 +800,6 @@ static struct _PyAptPkgAPIStruct API = {
    &PyHashString_Type,        // hashstring_type
    &PyHashString_FromCpp,     // hashstring_fromcpp
    &PyHashString_ToCpp,       // hashstring_tocpp
-   &PyIndexRecords_Type,      // indexrecords_type
-   &PyIndexRecords_FromCpp,     // indexrecords_tocpp
-   &PyIndexRecords_ToCpp,     // indexrecords_tocpp
    &PyMetaIndex_Type,         // metaindex_type
    &PyMetaIndex_FromCpp,        // metaindex_tocpp
    &PyMetaIndex_ToCpp,        // metaindex_tocpp
@@ -650,6 +818,7 @@ static struct _PyAptPkgAPIStruct API = {
    &PyPackageManager_Type,    // packagemanager_type
    &PyPackageManager_FromCpp,   // packagemanager_type
    &PyPackageManager_ToCpp,   // packagemanager_type
+   &PyPackageManagerProgressFancy_Type,
    &PyPackageRecords_Type,    // packagerecords_type
    0,                         // FIXME: packagerecords_fromcpp
    0,                         // FIXME: packagerecords_tocpp
@@ -666,10 +835,10 @@ static struct _PyAptPkgAPIStruct API = {
    0,                         // FIXME: sourcerecords_fromcpp
    0,                         // FIXME: sourcerecords_tocpp
    &PyTagFile_Type,           // tagfile_type
-   &PyTagFile_FromCpp,          // tagfile_tocpp
+   0,          // tagfile_tocpp
    &PyTagFile_ToCpp,          // tagfile_tocpp
    &PyTagSection_Type,        // tagsection_type
-   &PyTagSection_FromCpp,       // tagsection_tocpp
+   0,                         // tagsection_tocpp
    &PyTagSection_ToCpp,       // tagsection_tocpp
    &PyVersion_Type,           // version_type
    &PyVersion_FromCpp,          // version_tocpp
@@ -679,7 +848,10 @@ static struct _PyAptPkgAPIStruct API = {
    &PyGroup_ToCpp,            // group_tocpp
    &PyOrderList_Type,         // orderlist_type
    &PyOrderList_FromCpp,      // orderlist_fromcpp
-   &PyOrderList_ToCpp         // orderlist_tocpp
+   &PyOrderList_ToCpp,        // orderlist_tocpp
+   &PySourceRecordFiles_Type,     // sourcerecordfiles_type
+   0,                         // FIXME: sourcerecordfiles_fromcpp
+   0,                         // FIXME: sourcerecordfiles_tocpp
 };
 
 
@@ -719,6 +891,16 @@ extern "C" void initapt_pkg()
    // Finalize our types to add slots, etc.
    if (PyType_Ready(&PyConfiguration_Type) == -1) INIT_ERROR;
    if (PyType_Ready(&PyCacheFile_Type) == -1) INIT_ERROR;
+   PyAptError = PyErr_NewExceptionWithDoc("apt_pkg.Error", PyAptError_Doc, PyExc_SystemError, NULL);
+   if (PyAptError == NULL)
+      INIT_ERROR;
+   PyAptWarning = PyErr_NewExceptionWithDoc("apt_pkg.Warning", PyAptWarning_Doc, PyExc_Warning, NULL);
+   if (PyAptWarning == NULL)
+      INIT_ERROR;
+
+   PyAptCacheMismatchError = PyErr_NewExceptionWithDoc("apt_pkg.CacheMismatchError", PyAptCacheMismatchError_Doc, PyExc_ValueError, NULL);
+   if (PyAptCacheMismatchError == NULL)
+      INIT_ERROR;
 
    // Initialize the module
    #if PY_MAJOR_VERSION >= 3
@@ -733,7 +915,9 @@ extern "C" void initapt_pkg()
    // Global configuration, should never be deleted.
    Config->NoDelete = true;
    PyModule_AddObject(Module,"config",Config);
-
+   PyModule_AddObject(Module,"Error",PyAptError);
+   PyModule_AddObject(Module,"Warning",PyAptWarning);
+   PyModule_AddObject(Module,"CacheMismatchError", PyAptCacheMismatchError);
 
 
 
@@ -741,6 +925,10 @@ extern "C" void initapt_pkg()
    /* ============================ tag.cc ============================ */
    ADDTYPE(Module,"TagSection",&PyTagSection_Type);
    ADDTYPE(Module,"TagFile",&PyTagFile_Type);
+   ADDTYPE(Module,"Tag",&PyTag_Type);
+   ADDTYPE(Module,"TagRewrite",&PyTagRewrite_Type);
+   ADDTYPE(Module,"TagRename",&PyTagRename_Type);
+   ADDTYPE(Module,"TagRemove",&PyTagRemove_Type);
    /* ============================ acquire.cc ============================ */
    ADDTYPE(Module,"Acquire",&PyAcquire_Type);
    ADDTYPE(Module,"AcquireFile",&PyAcquireFile_Type);
@@ -772,13 +960,15 @@ extern "C" void initapt_pkg()
    /* ========================= pkgmanager.cc ========================= */
    ADDTYPE(Module,"_PackageManager",&PyPackageManager_Type);
    ADDTYPE(Module,"PackageManager",&PyPackageManager2_Type);
+   /* ========================= pkgmanagerprogress.cc ==================== */
+   ADDTYPE(Module,"PackageManagerProgressFancy",&PyPackageManagerProgressFancy_Type);
    /* ========================= pkgrecords.cc ========================= */
    ADDTYPE(Module,"PackageRecords",&PyPackageRecords_Type);
    /* ========================= pkgsrcrecords.cc ========================= */
    ADDTYPE(Module,"SourceRecords",&PySourceRecords_Type);
+   ADDTYPE(Module,"SourceRecordFiles",&PySourceRecordFiles_Type);
    /* ========================= sourcelist.cc ========================= */
    ADDTYPE(Module,"SourceList",&PySourceList_Type);
-   ADDTYPE(Module,"IndexRecords",&PyIndexRecords_Type);
    ADDTYPE(Module,"HashString",&PyHashString_Type);
    ADDTYPE(Module,"Policy",&PyPolicy_Type);
    ADDTYPE(Module,"Hashes",&PyHashes_Type);
@@ -786,6 +976,7 @@ extern "C" void initapt_pkg()
    ADDTYPE(Module,"SystemLock",&PySystemLock_Type);
    ADDTYPE(Module,"FileLock",&PyFileLock_Type);
    ADDTYPE(Module,"OrderList",&PyOrderList_Type);
+   ADDTYPE(Module,"HashStringList",&PyHashStringList_Type);
    // Tag file constants
    PyModule_AddObject(Module,"REWRITE_PACKAGE_ORDER",
                       CharCharToList(TFRewritePackageOrder));
@@ -841,9 +1032,11 @@ extern "C" void initapt_pkg()
    PyDict_SetItemString(PyPackageManager_Type.tp_dict, "RESULT_INCOMPLETE",
                         MkPyNumber(pkgPackageManager::Incomplete));
 
-
+   PyDict_SetItemString(PyVersion_Type.tp_dict, "MULTI_ARCH_NO",
+                        MkPyNumber(pkgCache::Version::No));
+   // NONE is deprecated (#782802)
    PyDict_SetItemString(PyVersion_Type.tp_dict, "MULTI_ARCH_NONE",
-                        MkPyNumber(pkgCache::Version::None));
+                        MkPyNumber(pkgCache::Version::No));
    PyDict_SetItemString(PyVersion_Type.tp_dict, "MULTI_ARCH_ALL",
                         MkPyNumber(pkgCache::Version::All));
    PyDict_SetItemString(PyVersion_Type.tp_dict, "MULTI_ARCH_FOREIGN",
@@ -869,7 +1062,13 @@ extern "C" void initapt_pkg()
                         MkPyNumber(pkgAcquire::Item::StatError));
    PyDict_SetItemString(PyAcquireItem_Type.tp_dict, "STAT_AUTH_ERROR",
                         MkPyNumber(pkgAcquire::Item::StatAuthError));
-
+   // TagSection constants
+   PyDict_SetItemString(PyTag_Type.tp_dict, "REMOVE",
+                        MkPyNumber(pkgTagSection::Tag::REMOVE));
+   PyDict_SetItemString(PyTag_Type.tp_dict, "REWRITE",
+                        MkPyNumber(pkgTagSection::Tag::REWRITE));
+   PyDict_SetItemString(PyTag_Type.tp_dict, "RENAME",
+                        MkPyNumber(pkgTagSection::Tag::RENAME));
 
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 1
    PyObject *PyCapsule = PyCapsule_New(&API, "apt_pkg._C_API", NULL);
@@ -880,8 +1079,13 @@ extern "C" void initapt_pkg()
    // Version..
    PyModule_AddStringConstant(Module,"VERSION",(char *)pkgVersion);
    PyModule_AddStringConstant(Module,"LIB_VERSION",(char *)pkgLibVersion);
-   PyModule_AddStringConstant(Module,"DATE",__DATE__);
-   PyModule_AddStringConstant(Module,"TIME",__TIME__);
+#ifdef DATE
+   PyModule_AddStringConstant(Module,"DATE",DATE);
+   PyModule_AddStringConstant(Module,"TIME",TIME);
+#else
+   PyModule_AddStringConstant(Module,"DATE", "Jan  1 1970");
+   PyModule_AddStringConstant(Module,"TIME", "00:00:00");
+#endif
 
    // My constants
    PyModule_AddIntConstant(Module,"PRI_IMPORTANT",pkgCache::State::Important);
@@ -908,7 +1112,6 @@ extern "C" void initapt_pkg()
    PyModule_AddIntConstant(Module,"INSTSTATE_HOLD",pkgCache::State::Hold);
    PyModule_AddIntConstant(Module,"INSTSTATE_HOLD_REINSTREQ",pkgCache::State::HoldReInstReq);
 
-
    #if PY_MAJOR_VERSION >= 3
    return Module;
    #endif
